package info.textgrid.middleware;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import javax.xml.datatype.DatatypeConfigurationException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.search.SearchHit;

/**
 * <p>
 * The Dublin Core Field Loader takes the results of the 
 * ElasticSearch request and puts them into a String-List. 
 * For each element of Dublin Core exists a seperate list. 
 * 
 * For each element two functions are necessary because the datatype "SeachHit" 
 * could contain more then one results and the GetResponse just one
 * </p>
 * 
 * @author Maximilian Brodhun, SUB Goettingen
 * @author Stefan E. Funk, SUB Goettingen
 * @version 24.05.2019
 * @since 18.02.2014
 */
public class DublinCoreFieldLoader {

  private static Log log = LogFactory.getLog(OAIPMHImpl.class);

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setContributor(SearchHit hit, String[] fields) {

    List<String> contributors = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        contributors.add(hit.getFields().get(field).values().get(0)
            .toString());
      }
    }

    return contributors;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setContributor(GetResponse responseWorkValues, String[] fields) {

    List<String> contributors = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          contributors.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return contributors;
  }


  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setCreator(GetResponse responseWorkValues, String[] fields) {

    List<String> creators = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          creators.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return creators;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setCreator(SearchHit hit, String[] fields) {

    List<String> creators = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        creators.add(hit.getFields().get(field).values().get(0)
            .toString());
      }
    }

    return creators;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setCoverage(GetResponse responseWorkValues, String[] fields) {

    List<String> coverages = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          coverages.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return coverages;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setCoverage(SearchHit hit, String[] fields) {

    List<String> coverages = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        coverages.add(hit.getFields().get(field).values().get(0)
            .toString());
      }
    }

    return coverages;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setDate(GetResponse responseWorkValues, String[] fields) {

    List<String> dates = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        try {
          // FIXME DO SET VALUES VIA CONFIG FILE!
          if (responseWorkValues.getField(field) != null) {
            dates.add(OAIPMHUtillities
                .convertDateFormat(responseWorkValues.getField(field).getValue().toString())
                .toXMLFormat());
          }
        } catch (ParseException e) {
          // FIXME DO NOT USE INVALID DATE HERE!
          dates.add(responseWorkValues.getField(field).getValue().toString());
          log.debug(e);
          log.debug("Tried to parse date value: "
              + responseWorkValues.getField(field).getValue().toString());
        } catch (DatatypeConfigurationException e) {
          // FIXME DO NOT USE INVALID DATE HERE!
          dates.add(responseWorkValues.getField(field).getValue().toString());
          log.debug("Tried to configure date value: "
              + responseWorkValues.getField(field).getValue().toString());
          log.debug(e);
        }
      }
    }

    return dates;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setDate(SearchHit hit, String[] fields) {

    List<String> dates = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {

        try {
          dates.add(OAIPMHUtillities.convertDateFormat(
              hit.getFields().get(field).values().get(0).toString()).toXMLFormat());
        } catch (ParseException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        } catch (DatatypeConfigurationException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    }

    return dates;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setDescription(GetResponse responseWorkValues, String[] fields) {

    List<String> descriptions = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          descriptions.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return descriptions;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setDescription(SearchHit hit, String[] fields) {

    List<String> descriptions = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        descriptions.add(hit.getFields().get(field).values().get(0)
            .toString());
      }
    }

    return descriptions;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setFormat(SearchHit hit, String[] fields) {

    List<String> formats = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        formats.add(hit.getFields().get(field).values().get(0)
            .toString());
      }
    }

    return formats;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setFormat(GetResponse responseWorkValues, String[] fields) {

    List<String> formats = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          formats.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return formats;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setIdentifier(SearchHit hit, String[] fields) {

    List<String> identifiers = new ArrayList<String>();

    String identifier = "";
    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        for (int i = 0; i < hit.getFields().get(field).getValues().size(); i++) {
          if (hit.getFields().get(field).values().get(i).toString().startsWith("textgrid")) {
            identifier = hit.getFields().get(field).values().get(i).toString();
            identifier = identifier.replace(identifier, "https://textgridrep.org/" + identifier);
            identifiers.add(identifier);
          } else {
            identifiers.add(hit.getFields().get(field).values().get(i).toString());
          }
        }
      }
    }

    return identifiers;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setIdentifier(GetResponse responseWorkValues, String[] fields) {

    List<String> identifiers = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {

        if (responseWorkValues.getField(field) != null) {
          List<Object> entireIdentifier = responseWorkValues.getField(field).getValues();
          for (int i = 0; i < responseWorkValues.getField(field).getValues().size(); i++) {
            identifiers.add(entireIdentifier.get(i).toString());
          }
        }
      }
    }

    return identifiers;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setLanguage(SearchHit hit, String[] fields) {

    List<String> languages = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        languages.add(hit.getFields().get(field).values().get(0).toString());
      }
    }

    return languages;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setLanguage(GetResponse responseWorkValues, String[] fields) {

    List<String> languages = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          languages.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return languages;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setPublisher(SearchHit hit, String[] fields) {

    List<String> publishers = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        publishers.add(hit.getFields().get(field).values().get(0).toString());
      }
    }

    return publishers;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setPublisher(GetResponse responseWorkValues, String[] fields) {

    List<String> publishers = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          publishers.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return publishers;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setRelation(SearchHit hit, String[] fields) {

    List<String> relations = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        for (int i = 0; i < hit.getFields().get(field).getValues().size(); i++) {
          relations.add(hit.getFields().get(field).values().get(i).toString());
        }
      }
    }

    return relations;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setRelation(GetResponse responseWorkValues, String[] fields) {

    List<String> relations = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          relations.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return relations;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setRelationForWork(GetResponse responseWorkValues, String[] fields) {

    List<String> relations = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          relations.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return relations;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setRights(SearchHit hit, String[] fields) {

    List<String> rights = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        rights.add(hit.getFields().get(field).values().get(0).toString());
      }
    }

    return rights;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setRights(GetResponse responseWorkValues, String[] fields) {

    List<String> rights = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          rights.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return rights;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setSources(SearchHit hit, String[] fields) {

    List<String> sources = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        sources.add(hit.getFields().get(field).values().get(0).toString());
      }
    }

    return sources;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setSources(GetResponse responseWorkValues, String[] fields) {

    List<String> sources = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        if (responseWorkValues.getField(field) != null) {
          sources.add(responseWorkValues.getField(field).getValue().toString());
        }
      }
    }

    return sources;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setSubject(GetResponse responseWorkValues, String[] fields) {

    List<String> subjects = new ArrayList<String>();

    for (String field : fields) {
      if (responseWorkValues.getField(field) != null) {
        subjects.add(responseWorkValues.getField(field).getValue().toString());
      }
    }

    return subjects;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setSubject(SearchHit hit, String[] fields) {

    List<String> subjects = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        subjects.add(hit.getFields().get(field).values().get(0).toString());
      }
    }

    return subjects;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setTitle(SearchHit hit, String[] fields) {

    List<String> titles = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        for (int i = 0; i < hit.getFields().get(field).getValues().size(); i++) {
          titles.add(hit.getFields().get(field).values().get(i).toString());
        }
      }
    }

    return titles;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setTitle(GetResponse responseWorkValues, String[] fields) {

    List<String> titles = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        List<Object> entireTitles = responseWorkValues.getField(field).getValues();
        if (responseWorkValues.getField(field) != null) {
          for (int i = 0; i < responseWorkValues.getField(field).getValues().size(); i++) {
            titles.add(entireTitles.get(i).toString());
          }
        }
      }
    }

    return titles;
  }

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setType(GetResponse responseWorkValues, String[] fields) {

    List<String> types = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      if (fields != null) {
        for (String field : fields) {
          if (responseWorkValues.getField(field) != null) {
            types.add(responseWorkValues.getField(field).getValue().toString());
          }
        }
      }
    }

    return types;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setType(SearchHit hit, String[] fields) {

    List<String> types = new ArrayList<String>();

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        for (int i = 0; i < hit.getFields().get(field).getValues().size(); i++) {
          types.add(hit.getFields().get(field).values().get(i).toString());
        }
      }
    }

    return types;
  }

}
